# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2016-12-19 10:41
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('layers', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='layer',
            options={'ordering': ('name',)},
        ),
    ]
